@@ -190,3 +190,12 @@ class AnyscaleAWSNodeProvider(AWSNodeProvider):  # type: ignore
         delete_credentials_from_config(cluster_config.get("provider", {}))
 
         return cluster_config
+
+    def describe_instances(self, *args: Any, **kwargs: Any) -> Any:
+        return self.ec2.meta.client.describe_instances(*args, **kwargs)
+
+    def start_nodes(self, node_ids: List[str]) -> Any:
+        return self.ec2.meta.client.start_instances(InstanceIds=node_ids)
+
+    def stop_nodes(self, node_ids: List[str]) -> Any:
+        return self.ec2.meta.client.stop_instances(InstanceIds=node_ids)
