@@ -10,13 +10,11 @@ from typing import cast, Dict, Iterator, List, Optional
 
 import click
 from openapi_client.rest import ApiException  # type: ignore
-import ray
 
 import anyscale
 from anyscale.client.openapi_client.api.default_api import DefaultApi  # type: ignore
 from anyscale.util import format_api_exception, get_rsync_command, send_json_request
 
-logging.basicConfig(format=ray.ray_constants.LOGGER_FORMAT)
 logger = logging.getLogger(__file__)
 logging.getLogger("botocore").setLevel(logging.CRITICAL)
 
