@@ -113,8 +113,10 @@ def _update_file_mounts(
     project_id = get_project_id(project_root_dir)
     working_dir = get_working_dir(cluster_config, project_id, api_client)
     for remote_path, local_path in cluster_config.get("file_mounts", {}).items():
-        if remote_path == working_dir and not os.path.samefile(
-            local_path, project_root_dir
+        if (
+            remote_path == working_dir
+            and os.path.exists(local_path)
+            and not os.path.samefile(local_path, project_root_dir)
         ):
             click.confirm(
                 '"{remote}: {local}" has been detected in the file mounts.\n'
@@ -203,7 +205,9 @@ def configure_for_session(
 
 
 def get_cluster_config(
-    session_name: Optional[str], api_client: DefaultApi = None
+    session_name: Optional[str],
+    api_client: DefaultApi = None,
+    disable_project_sync: bool = False,
 ) -> Dict[str, Any]:
 
     project_definition = load_project_or_throw()
@@ -212,7 +216,10 @@ def get_cluster_config(
     session = get_project_session(project_id, session_name, api_client)
 
     return configure_for_session(
-        session.id, project_definition.root, api_client=api_client
+        session.id,
+        project_definition.root,
+        api_client=api_client,
+        disable_project_sync=disable_project_sync,
     )
 
 
