@@ -12,12 +12,10 @@ from typing import Any, cast, Dict, List, Optional, Tuple, Union
 import uuid
 
 import colorama  # type: ignore
-import ray
-import ray.autoscaler.sdk
 import requests
 import yaml
 
-from anyscale.client.openapi_client.models.application_template import ApplicationTemplate  # type: ignore
+from anyscale.client.openapi_client.models.app_config import AppConfig  # type: ignore
 from anyscale.client.openapi_client.models.build import Build  # type: ignore
 from anyscale.client.openapi_client.models.session import Session  # type: ignore
 from anyscale.credentials import load_credentials
@@ -37,17 +35,55 @@ MAX_SESSIONS = 20
 # Default minutes for autosuspend.
 DEFAULT_AUTOSUSPEND_TIMEOUT = 120
 
-# Last updated 2/23/21
+# The required Ray version on the client side.
+REQUIRED_RAY_VERSION = "2.0.0.dev0"
+REQUIRED_RAY_COMMIT = "63d77e0d4c63064ed0ee43adc5a2a71ccc16254a"
+
+# Pin images to use on the server side.
 PINNED_IMAGES = {
-    "anyscale/ray-ml:nightly-py36-cpu": "anyscale/ray-ml@sha256:a2726aaffe167a13178192c4d614d727994ab4f3bd9cb03086b4b3309cb8b36f",
-    "anyscale/ray-ml:nightly-py36-gpu": "anyscale/ray-ml@sha256:ebe07a5a0974fa9153de412259d985d514f9e1891bab1ee2ae4c5c890d704794",
-    "anyscale/ray-ml:nightly-py37-cpu": "anyscale/ray-ml@sha256:fd5b85241c0876971288deecc8d014969b791faa5e514ea7f667c68a56450179",
-    "anyscale/ray-ml:nightly-py37-gpu": "anyscale/ray-ml@sha256:017d1959b124ad38af0f2b8cce7de500e947e28bf307eb340a955d07d46c8ebc",
-    "anyscale/ray-ml:nightly-py38-cpu": "anyscale/ray-ml@sha256:c8e837d96fcf3f370ac468fa1744a30c7793f35b1bc6c9e7b41429c524a734ff",
-    "anyscale/ray-ml:nightly-py38-gpu": "anyscale/ray-ml@sha256:2011eeb1ba5d05da67f49b4df66fe4f36b72e45ea0960bcf94a38d4f112e3f68",
+    "anyscale/ray-ml:nightly-py36-cpu": "anyscale/ray-ml@sha256:709e7c004818fee40374dd952da2a02ef2a94f767c5ce9f6c80124ed498b323a",
+    "anyscale/ray-ml:nightly-py36-gpu": "anyscale/ray-ml@sha256:6677b48a7c0b34c6f258db5e1bf1b9aafffbd68700a98d7a8ded00f70c87d730",
+    "anyscale/ray-ml:nightly-py37-cpu": "anyscale/ray-ml@sha256:53dca8deb652be7ba46276b3436aae2b2979e757195fe66cad6829d1f10f7e47",
+    "anyscale/ray-ml:nightly-py37-gpu": "anyscale/ray-ml@sha256:3f658a9178fe8c8981cdab8bd1da199daac01635549eab85c88aeee4b33b8a96",
+    "anyscale/ray-ml:nightly-py38-cpu": "anyscale/ray-ml@sha256:8f9eb6274e488f27cef2eec0897095387e9b6d0006b3a516966565e43489f914",
+    "anyscale/ray-ml:nightly-py38-gpu": "anyscale/ray-ml@sha256:",
 }
 
 
+def _get_wheel_url(
+    ray_release: str,
+    py_version: Optional[str] = None,
+    sys_platform: Optional[str] = None,
+) -> str:
+    """Return S3 URL for the given release spec or 'latest'."""
+    if py_version is None:
+        py_version = "".join(str(x) for x in sys.version_info[0:2])
+    if sys_platform is None:
+        sys_platform = sys.platform
+
+    if sys_platform == "darwin":
+        if py_version == "38":
+            platform = "macosx_10_13_x86_64"
+        else:
+            platform = "macosx_10_13_intel"
+    elif sys_platform == "win32":
+        platform = "win_amd64"
+    else:
+        platform = "manylinux2014_x86_64"
+
+    if py_version == "38":
+        py_version_malloc = py_version
+    else:
+        py_version_malloc = f"{py_version}m"
+
+    return (
+        "https://s3-us-west-2.amazonaws.com/ray-wheels/"
+        "{}/ray-2.0.0.dev0-cp{}-cp{}-{}.whl".format(
+            ray_release, py_version, py_version_malloc, platform
+        )
+    )
+
+
 # Default docker images to use for connect sessions.
 def _get_base_image(cpu_or_gpu: str = "cpu") -> str:
     py_version = "".join(str(x) for x in sys.version_info[0:2])
@@ -86,7 +122,7 @@ class _ConsoleLog:
         print(*msg)
 
     def debug(self, *msg: str) -> None:
-        if os.environ.get("DEBUG") == "1":
+        if os.environ.get("ANYSCALE_DEBUG") == "1":
             print(
                 "{}{}(anyscale +{}){} ".format(
                     colorama.Style.DIM,
@@ -175,13 +211,17 @@ class SessionBuilder:
         anyscale_sdk: AnyscaleSDK = None,
         subprocess: Any = subprocess,
         requests: Any = requests,
-        ray: Any = ray,
+        _ray: Any = None,
         log: Any = _ConsoleLog(),
         _os: Any = os,
+        _in_shell: Any = _in_shell,
+        _ignore_version_check: bool = False,
     ) -> None:
+        if "IGNORE_VERSION_CHECK" in os.environ:
+            _ignore_version_check = True
         # Class dependencies.
         self._log = log
-        self._anyscale_sdk: Any = None
+        self._anyscale_sdk: AnyscaleSDK = None
         self._credentials = None
         if anyscale_sdk:
             self._anyscale_sdk = anyscale_sdk
@@ -192,10 +232,16 @@ class SessionBuilder:
             self._anyscale_sdk = AnyscaleSDK(
                 self._credentials, os.path.join(anyscale.conf.ANYSCALE_HOST, "ext")
             )
-        self._ray: Any = ray
+        if not _ray:
+            import ray
+
+            _ray = ray
+        self._ray: Any = _ray
         self._subprocess: Any = subprocess
         self._os: Any = _os
         self._requests: Any = requests
+        self._in_shell: Any = _in_shell
+        self._ignore_version_check = _ignore_version_check
 
         # Builder args.
         self._scratch_dir: str = scratch_dir
@@ -236,6 +282,10 @@ class SessionBuilder:
         Examples:
             >>> anyscale.connect()
         """
+        import ray
+        import ray.autoscaler.sdk
+
+        self._check_required_ray_version(ray.__version__, ray.__commit__)
 
         # TODO(ekl) check for duplicate connections
         self._log.zero_time()
@@ -253,7 +303,7 @@ class SessionBuilder:
 
         # Re-exec in the docker container.
         if self._run_mode == "local_docker":
-            if _in_shell():
+            if self._in_shell():
                 raise ValueError("Local docker mode is not supported in Python shells.")
             self._exec_self_in_local_docker()
 
@@ -262,19 +312,38 @@ class SessionBuilder:
             self._project_dir = anyscale.project.find_project_root(os.getcwd())
         if self._project_dir:
             self._ensure_project_setup_at_dir(self._project_dir, self._project_name)
-        else:
+        elif self._in_shell():
             self._project_dir = self._get_or_create_scratch_project()
+        else:
+            raise ValueError(
+                "The current working directory is not associated with an "
+                "Anyscale project. Please run ``anyscale init`` to setup a "
+                "new project or specify ``.project_dir()`` prior to "
+                "connecting. Anyscale Connect uses the project directory "
+                "to find the Python code dependencies for your script."
+            )
         proj_def = anyscale.project.ProjectDefinition(self._project_dir)
         project_id = anyscale.project.get_project_id(proj_def.root)
         self._log.info("Using project dir", proj_def.root)
 
         # TODO(ekl): generate a serverless compute configuraton here.
         cluster_yaml = yaml.safe_load(anyscale.project.CLUSTER_YAML_TEMPLATE)
+
+        build = (
+            self._get_app_config_build(
+                project_id, self._app_config_name, self._app_config_revision
+            )
+            if self._app_config_name
+            else None
+        )
+
         self._populate_cluster_config(
             cluster_yaml,
             project_id,
             self._anyscale_sdk.get_project(project_id).result.name,
+            build=build,
         )
+
         with open(os.path.join(self._project_dir, "session-default.yaml"), "w+") as f:
             f.write(yaml.dump(cluster_yaml))
 
@@ -295,7 +364,10 @@ class SessionBuilder:
         # Locate a updated session and update it if needed.
         if self._session_name is None or needs_up:
             self._session_name = self._get_or_create_updated_session(
-                files_hash.encode("utf-8"), project_id, self._session_name
+                files_hash.encode("utf-8"),
+                project_id,
+                self._session_name,
+                build_id=build.id if build else None,
             )
 
         # Connect Ray client.
@@ -322,7 +394,7 @@ class SessionBuilder:
 
         # If in background mode, execute the job in the remote session.
         if self._run_mode == "background":
-            if _in_shell():
+            if self._in_shell():
                 raise ValueError("Background mode is not supported in Python shells.")
             self._exec_self_in_head_node()
 
@@ -416,6 +488,9 @@ class SessionBuilder:
     def base_docker_image(self, image_name: str) -> "SessionBuilder":
         """Set the docker image to use for the session.
 
+        IMPORTANT: the Python minor version of the manually specified docker
+        image must match the local Python version.
+
         Args:
             image_name (str): docker image name.
 
@@ -449,6 +524,9 @@ class SessionBuilder:
     def app_config(self, app_config_identifier: str) -> "SessionBuilder":
         """Set the Anyscale app config to use for the session.
 
+        IMPORTANT: the Python minor version of the manually specified app
+        config must match the local Python version.
+
         Args:
             app_config_identifier (str): Name (and optionally revision) of
             the application config, for example "my_app_config:2" where the
@@ -552,7 +630,7 @@ class SessionBuilder:
         if len(git_commit) != 40:
             raise ValueError("Ray git commit hash must be 40 chars long")
         self._ray_release = "master/{}".format(git_commit)
-        url = self._get_wheel_url(self._ray_release)
+        url = _get_wheel_url(self._ray_release)
         request = self._requests.head(url)
         if request.status_code != 200:
             raise ValueError(
@@ -660,7 +738,12 @@ class SessionBuilder:
                 f.write(yaml.dump({"project_id": project_id}))
 
     def _up_session(
-        self, project_id: str, session_name: str, cloud_name: str, cwd: Optional[str]
+        self,
+        project_id: str,
+        session_name: str,
+        cloud_name: str,
+        cwd: Optional[str],
+        dangerously_set_build_id: Optional[str],
     ) -> None:
         # Non-blocking version of check_call, see
         # https://github.com/python/cpython/blob/64abf373444944a240274a9b6d66d1cb01ecfcdd/Lib/subprocess.py#L363
@@ -674,7 +757,11 @@ class SessionBuilder:
             "--cloud-name",
             cloud_name,
             session_name,
-        ]
+        ] + (
+            ["--dangerously-set-build-id", dangerously_set_build_id]
+            if dangerously_set_build_id
+            else []
+        )
         with tempfile.NamedTemporaryFile(delete=False) as output_log_file:
             with self._subprocess.Popen(
                 command, cwd=cwd, stdout=output_log_file, stderr=subprocess.STDOUT,
@@ -749,7 +836,11 @@ class SessionBuilder:
         return cast(str, cloud_name)
 
     def _get_or_create_updated_session(  # noqa: C901
-        self, files_hash: bytes, project_id: str, session_name: Optional[str]
+        self,
+        files_hash: bytes,
+        project_id: str,
+        session_name: Optional[str],
+        build_id: Optional[str],
     ) -> str:
         """Get or create a session updated with the given hash.
 
@@ -779,7 +870,11 @@ class SessionBuilder:
                 )
                 if self._tmp_conn is None:
                     continue
-                session_hash = self._tmp_conn._internal_kv_get(ANYSCALE_FILES_HASH)
+                try:
+                    session_hash = self._tmp_conn._internal_kv_get(ANYSCALE_FILES_HASH)
+                except Exception as e:
+                    self._log.debug("Error getting session hash", e)
+                    session_hash = None
                 if session_hash == files_hash:
                     self._log.debug("Acquired lock on session", session.name)
                     session_name = session.name
@@ -842,7 +937,11 @@ class SessionBuilder:
                 self._tmp_conn = None
             # Update session.
             self._up_session(
-                project_id, session_name, self._cloud_name, self._project_dir
+                project_id,
+                session_name,
+                self._cloud_name,
+                self._project_dir,
+                dangerously_set_build_id=build_id,
             )
             # Need to re-acquire the connection after the update.
             self._tmp_conn = self._acquire_session_lock(
@@ -915,7 +1014,8 @@ class SessionBuilder:
             return "Connected!"
 
         f_remote = self._ray.remote(func)
-        self._log.debug(self._ray.get(f_remote.remote()))
+        ray_ref = f_remote.remote()
+        self._log.debug(self._ray.get(ray_ref))
         self._log.info(
             "Connected to {}, see: {}/o/anyscale-internal/"
             "projects/{}/sessions/{}".format(
@@ -974,7 +1074,11 @@ class SessionBuilder:
         return fingerprint_hasher.hexdigest()
 
     def _populate_cluster_config(
-        self, cluster_yaml: Dict[str, Any], project_id: str, project_name: str
+        self,
+        cluster_yaml: Dict[str, Any],
+        project_id: str,
+        project_name: str,
+        build: Optional[Build],
     ) -> None:
         """Populate cluster config with serverless compute options."""
 
@@ -982,12 +1086,10 @@ class SessionBuilder:
         base_docker_image_gpu = _get_base_image("gpu")
 
         # Overwrite base_docker_image with application config if available.
-        if self._app_config_name:
+        if build:
             base_docker_image_cpu = (
                 base_docker_image_gpu
-            ) = self._get_app_config_docker_image(
-                project_id, self._app_config_name, self._app_config_revision
-            )
+            ) = self._get_app_config_docker_image(project_id, build_to_use=build)
         elif self._base_docker_image:
             # Overwrite base_docker_image if provided by user.
             base_docker_image_cpu = self._base_docker_image
@@ -1023,13 +1125,12 @@ class SessionBuilder:
         }
         cluster_yaml["max_workers"] = 50
         cluster_yaml["head_node_type"] = "anyscale.cpu.medium"
-        cluster_yaml["worker_default_node_type"] = "anyscale.cpu.medium"
         cluster_yaml["docker"]["image"] = base_docker_image_cpu
 
         # Install Ray runtime if specified.
         if self._ray_release:
             cluster_yaml["setup_commands"] = [
-                "pip install -U {}".format(self._get_wheel_url(self._ray_release))
+                "pip install -U {}".format(_get_wheel_url(self._ray_release))
             ]
 
         # Install requirements:
@@ -1044,21 +1145,12 @@ class SessionBuilder:
             "ulimit -n 65536; cd ~/"
             + project_name
             + "; AUTOSCALER_UPDATE_INTERVAL_S=5 ray start --head --port=6379 "
-            "--object-manager-port=8076 "
+            f"--object-manager-port=8076 --gcs-server-port={anyscale.conf.RAY_STATIC_GCS_PORT} "
             "--ray-client-server-port=10001 "
             "--autoscaling-config=~/ray_bootstrap_config.yaml",
         ]
 
-    def _get_wheel_url(self, ray_release: str) -> str:
-        """Return S3 URL for the given release spec or 'latest'."""
-        return (
-            "https://s3-us-west-2.amazonaws.com/ray-wheels/"
-            "{}/ray-2.0.0.dev0-cp37-cp37m-manylinux2014_x86_64.whl".format(
-                self._ray_release
-            )
-        )
-
-    def _wait_for_app_build(self, project_id: str, build_id: str) -> None:
+    def _wait_for_app_build(self, project_id: str, build_id: str) -> Build:
         has_logged = False
         while True:
             build = self._anyscale_sdk.get_build(build_id).result
@@ -1078,13 +1170,13 @@ class SessionBuilder:
                 )
             else:
                 assert build.status == "succeeded"
-                break
+                return build
 
-    def _get_app_config_docker_image(
+    def _get_app_config_build(
         self, project_id: str, app_config_name: str, app_config_revision: Optional[int]
-    ) -> str:
+    ) -> Build:
         app_template_id = None
-        app_templates = self._list_application_templates(project_id)
+        app_templates = self._list_app_configs(project_id)
         for app_template in app_templates:
             if app_template.name == app_config_name:
                 app_template_id = app_template.id
@@ -1121,9 +1213,12 @@ class SessionBuilder:
                 )
             )
         assert build_to_use  # for mypy
+        return build_to_use
+
+    def _get_app_config_docker_image(self, project_id: str, build_to_use: Build) -> str:
         # Wait for build to complete:
-        self._wait_for_app_build(project_id, build_to_use.id)
-        return "localhost:5556/{}".format(build_to_use.docker_image_name)
+        build_to_use = self._wait_for_app_build(project_id, build_to_use.id)
+        return "localhost:5555/{}".format(build_to_use.docker_image_name)
 
     def _exec_self_in_head_node(self) -> None:
         """Run the current main file in the head node."""
@@ -1205,12 +1300,38 @@ class SessionBuilder:
     def _list_sessions(self, project_id: str) -> List[Session]:
         return self._list_entities(self._anyscale_sdk.list_sessions, project_id)
 
-    def _list_application_templates(self, project_id: str) -> List[ApplicationTemplate]:
-        return self._list_entities(
-            self._anyscale_sdk.list_application_templates, project_id
-        )
+    def _list_app_configs(self, project_id: str) -> List[AppConfig]:
+        return self._list_entities(self._anyscale_sdk.list_app_configs, project_id)
 
     def _list_builds(self, application_template_id: str) -> List[Build]:
         return self._list_entities(
             self._anyscale_sdk.list_builds, application_template_id
         )
+
+    def _check_required_ray_version(self, ray_version: str, ray_commit: str) -> None:
+        if ray_version != REQUIRED_RAY_VERSION or ray_commit != REQUIRED_RAY_COMMIT:
+            msg = (
+                "The local Ray installation has version {} (commit {}), but "
+                "{} (commit {}) is required. Please install the required "
+                "Ray version by running `pip uninstall ray -y && pip install -U {}`. To unsafely "
+                "ignore this check, set IGNORE_VERSION_CHECK=1.".format(
+                    ray_version,
+                    ray_commit,
+                    REQUIRED_RAY_VERSION,
+                    REQUIRED_RAY_COMMIT,
+                    _get_wheel_url("master/{}".format(REQUIRED_RAY_COMMIT)),
+                )
+            )
+            if self._ignore_version_check:
+                self._log.debug(msg)
+            else:
+                raise ValueError(msg)
+
+
+# This implements the following utility function for users:
+# $ pip install -U `python -m anyscale.connect required_ray_version`
+if __name__ == "__main__":
+    if len(sys.argv) > 1 and sys.argv[1] == "required_ray_version":
+        print(_get_wheel_url("master/{}".format(REQUIRED_RAY_COMMIT)))
+    else:
+        raise ValueError("Unsupported argument.")
