@@ -63,7 +63,9 @@ class CloudController(object):
 
         return True
 
-    def setup_cloud(self, provider: str, region: Optional[str], name: str) -> None:
+    def setup_cloud(
+        self, provider: str, region: Optional[str], name: str, yes: bool = False
+    ) -> None:
         """
         Sets up a cloud provider
         """
@@ -72,7 +74,7 @@ class CloudController(object):
             # If the region is blank, change it to the default for AWS.
             if region is None:
                 region = "us-west-2"
-            self.setup_aws(region=region, name=name)
+            self.setup_aws(region=region, name=name, yes=yes)
         elif provider == "gcp":
             # If the region is blank, change it to the default for GCP.
             if region is None:
@@ -84,7 +86,7 @@ class CloudController(object):
                 f"Invalid Cloud provider: {provider}. Available providers are [aws, gcp]."
             )
 
-    def setup_aws(self, region: str, name: str) -> None:
+    def setup_aws(self, region: str, name: str, yes: bool = False) -> None:
         from ray.autoscaler._private.aws.config import DEFAULT_RAY_IAM_ROLE
 
         os.environ["AWS_DEFAULT_REGION"] = region
@@ -97,7 +99,7 @@ class CloudController(object):
         confirm(
             "\nYou are about to give anyscale full access to EC2 and IAM in your AWS account.\n\n"
             "Continue?",
-            False,
+            yes,
         )
 
         with format_api_exception(ApiException):
