@@ -22,7 +22,7 @@ from anyscale.util import execution_log_name
 
 
 def rsync(*args: Any, **kwargs: Any) -> None:
-    from ray.autoscaler._private.commands import rsync as ray_rsync
+    from ray.autoscaler.sdk import rsync as ray_rsync
 
     ray_rsync(*args, **kwargs)
 
@@ -104,12 +104,7 @@ class ExecController(object):
             # Rsync file mounts if sync flag is set
             if sync:
                 rsync(
-                    config_file.name,
-                    source=None,
-                    target=None,
-                    override_cluster_name=None,
-                    down=False,
-                    all_nodes=True,
+                    cluster_config, source=None, target=None, down=False,
                 )
 
             command = [
