@@ -31,8 +31,8 @@ class ProjectController:
             api_client = get_api_client()
         self.api_client = api_client
 
-    def clone(self, project_name: str) -> None:
-        project_id = get_proj_id_from_name(project_name, self.api_client)
+    def clone(self, project_name: str, owner: Optional[str] = None) -> None:
+        project_id = get_proj_id_from_name(project_name, self.api_client, owner)
 
         os.makedirs(project_name)
         clone_cluster_config(project_name, project_name, project_id, self.api_client)
