@@ -1,17 +1,13 @@
-import copy
 from datetime import datetime
 import json
 import os
 import subprocess
-import sys
-import tempfile
 from typing import Any, Dict, Optional, Tuple, Union
 
 import click
 from openapi_client.rest import ApiException  # type: ignore
 import yaml
 
-from anyscale import ANYSCALE_ENV
 from anyscale.api import get_api_client
 from anyscale.autosync_heartbeat import (
     get_working_dir_host_mount_location,
@@ -24,7 +20,6 @@ from anyscale.client.openapi_client import (  # type: ignore
     CreateComputeTemplate,
     CreateSessionFromSnapshotOptions,
     Session,
-    SessionFinishUpOptions,
     SessionState,
     SessionUpOptions,
     SetupInitializeSessionOptions,
@@ -35,7 +30,6 @@ from anyscale.cloud import get_cloud_id_and_name
 from anyscale.cluster_config import (
     _configure_min_max_workers,
     _update_file_mounts,
-    configure_for_session,
     get_cluster_config,
 )
 from anyscale.project import (
@@ -46,12 +40,10 @@ from anyscale.project import (
 )
 from anyscale.util import (
     canonicalize_remote_location,
-    download_anyscale_wheel,
     format_api_exception,
     get_container_name,
     get_endpoint,
     get_working_dir,
-    install_anyscale_hooks,
     populate_session_args,
     slugify,
     validate_cluster_configuration,
@@ -85,7 +77,6 @@ class SessionController:
     def stop(
         self,
         session_name: Optional[str],
-        terminate: bool,
         delete: bool,
         workers_only: bool,
         keep_min_workers: bool,
@@ -94,7 +85,6 @@ class SessionController:
         project_definition = load_project_or_throw()
         project_id = get_project_id(project_definition.root)
         sessions = get_project_sessions(project_id, session_name, self.api_client)
-        terminate = terminate or delete
 
         if not session_name and len(sessions) > 1:
             raise click.ClickException(
@@ -109,7 +99,7 @@ class SessionController:
             self.api_client.stop_session_api_v2_sessions_session_id_stop_post(
                 session.id,
                 StopSessionOptions(
-                    terminate=terminate,
+                    terminate=True,
                     workers_only=workers_only,
                     keep_min_workers=keep_min_workers,
                     delete=delete,
@@ -120,7 +110,7 @@ class SessionController:
         session_names = [session.name for session in sessions]
         session_names_str = ", ".join(session_names)
         url = get_endpoint(f"/projects/{project_id}")
-        print(f"Session {session_names_str} stopping. View progress at {url}")
+        print(f"Session {session_names_str} shutting down. View progress at {url}")
 
     def _load_cluster_config(
         self, config: Optional[str], session_exists: bool, project_definition: Any
@@ -155,195 +145,6 @@ class SessionController:
         return cluster_config
 
     def up(
-        self,
-        session_name: Optional[str],
-        config: Optional[str],
-        build_id: Optional[str],
-        compute_template_id: Optional[str],
-        min_workers: Optional[int],
-        max_workers: Optional[int],
-        no_restart: bool,
-        restart_only: bool,
-        disable_sync: bool,
-        cloud_id: Optional[str],
-        cloud_name: Optional[str],
-        idle_timeout: Optional[int],
-        yes: bool,
-        verbose: bool = False,
-        no_rapid_start: bool = False,
-    ) -> None:
-        import ray.scripts.scripts as autoscaler_scripts
-
-        # TODO(ilr) Split this into ~3 functions, look at jbai's comments on #1469
-
-        project_definition = load_project_or_throw()
-        project_id = get_project_id(project_definition.root)
-
-        if not session_name:
-            session_name = str(
-                self.api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get(
-                    project_id=project_id,
-                ).result.name
-            )
-
-        session_list = self.api_client.list_sessions_api_v2_sessions_get(
-            project_id=project_id, active_only=False, name=session_name
-        ).results
-
-        session_exists = len(session_list) > 0
-
-        # Deduce whether we are launching this session
-        # from a cluster config yaml
-        # as opposed to a build ID + compute template ID combination.
-        # The logic is:
-        #  - If either build or compute template is specified,
-        #    we will use those. Both must be specified.
-        #  - Otherwise, we will use a cluster config yaml.
-        #     - If a yaml name is explicitly specified, we will use that.
-        #     - Otherwise, look for session-default.yaml.
-        using_cluster_config = build_id is None and compute_template_id is None
-        if not using_cluster_config:
-            if config:
-                raise ValueError(
-                    "If launching from a build/compute template, a config file cannot be specified."
-                )
-            if not build_id or not compute_template_id:
-                raise ValueError(
-                    "Both build ID and Compute Template must be specified if launching a session that way."
-                )
-
-        # If using a user cluster config, load it.
-        maybe_cluster_config = None
-        if using_cluster_config:
-            maybe_cluster_config = self._load_cluster_config(
-                config=config,
-                session_exists=session_exists,
-                project_definition=project_definition,
-            )
-
-        if no_rapid_start:
-            if not maybe_cluster_config:
-                raise ValueError(
-                    "Please specify a cluster config if using no_rapid_start."
-                )
-            maybe_cluster_config["provider"]["bypass_rapid_start"] = True
-
-        cloud_id, _ = get_cloud_id_and_name(self.api_client, cloud_id, cloud_name)
-        assert cloud_id is not None, "Failed to get cloud."
-
-        with format_api_exception(ApiException):
-            up_response = self.api_client.session_up_api_v2_sessions_up_post(
-                session_up_options=SessionUpOptions(
-                    project_id=project_id,
-                    name=session_name,
-                    cluster_config={"config": json.dumps(maybe_cluster_config)}
-                    if maybe_cluster_config
-                    else None,
-                    build_id=build_id,
-                    compute_template_id=compute_template_id,
-                    cloud_id=cloud_id,
-                    idle_timeout=idle_timeout,
-                )
-            ).result
-        session_id = up_response.session_id
-
-        # Download & write anyscale wheel!
-        download_anyscale_wheel(self.api_client, session_id)
-
-        # Get previous head IP
-        with format_api_exception(ApiException):
-            ip_resp = self.api_client.get_session_api_v2_sessions_session_id_get(
-                session_id
-            )
-        prev_head_ip = ip_resp.result.head_node_ip
-
-        cluster_config = up_response.cluster_config
-
-        if cluster_config["provider"].get("type") != "kubernetes":
-            cluster_config = configure_for_session(
-                session_id,
-                project_definition.root,
-                api_client=self.api_client,
-                disable_project_sync=disable_sync,
-                _DO_NOT_USE_RAY_UP_ONLY_cluster_config=cluster_config,
-            )
-
-        install_anyscale_hooks(cluster_config)
-
-        with tempfile.NamedTemporaryFile(mode="w") as config_file:
-            renamed_config = copy.deepcopy(cluster_config)
-            renamed_config["cluster_name"] = up_response.cluster_name
-            json.dump(renamed_config, config_file)
-            config_file.flush()
-            try:
-                # Use the bundled Autoscaler by directly calling the scripts file
-                command = [
-                    sys.executable,
-                    autoscaler_scripts.__file__,
-                    "up",
-                    "-vvvv" if verbose else "",
-                    config_file.name,
-                    "--no-restart" if no_restart else "",
-                    "--restart-only" if restart_only else "",
-                    "--yes",
-                ]
-                command_lst = [c for c in command if c]
-                if max_workers:
-                    command_lst.extend(["--max-workers", f"{max_workers}"])
-                if min_workers:
-                    command_lst.extend(["--min-workers", f"{min_workers}"])
-
-                # Set OS enviornment variables temporarily.
-                aws_credentials = get_credentials_as_env_vars_from_cluster_config(
-                    cluster_config
-                )
-
-                env = {
-                    **ANYSCALE_ENV,
-                    **aws_credentials,
-                }
-
-                proc = subprocess.Popen(
-                    command_lst,
-                    stdout=subprocess.PIPE,
-                    stderr=subprocess.STDOUT,
-                    env=env,
-                )
-                startup_log = []
-                while proc.stdout:
-                    line = proc.stdout.readline().decode()
-                    if not line:
-                        break
-                    print(line, end="")
-                    startup_log.append(line)
-                startup_log_str = "".join(startup_log)
-                proc.communicate()
-
-                ray_up_succeeded = proc.returncode == 0
-
-                curr_head_ip = get_head_node_ip(renamed_config)
-
-                if curr_head_ip != prev_head_ip and ray_up_succeeded:
-                    print("Setting up Jupyter lab, Ray dashboard, and autopush ...")
-
-                self.api_client.session_finish_up_api_v2_sessions_session_id_finish_up_post(
-                    session_id=session_id,
-                    session_finish_up_options=SessionFinishUpOptions(
-                        startup_log=startup_log_str,
-                        new_session=curr_head_ip != prev_head_ip,
-                        head_node_ip=curr_head_ip,
-                        cluster_launch_errored=not ray_up_succeeded,
-                    ),
-                )
-                # NOTE This call will raise an error if the session is in any of the following
-                # states (StartupErrored/Stopped/Terminated).
-                wait_for_session_start(project_id, session_name, self.api_client)
-                url = get_endpoint(f"/projects/{project_id}")
-                print(f"Session {session_name} started. View at {url}")
-            except Exception as e:
-                raise click.ClickException("{}\nSession startup failed.".format(e))
-
-    def up_internal(
         self,
         session_name: Optional[str],
         config: Optional[str],
@@ -359,6 +160,7 @@ class SessionController:
         idle_timeout: Optional[int],
         verbose: bool = False,
         no_rapid_start: bool = False,
+        dangerously_set_build_id: Optional[str] = None,
     ) -> None:
         # TODO(ilr) Split this into ~3 functions, look at jbai's comments on #1469
         project_definition = load_project_or_throw()
@@ -386,6 +188,7 @@ class SessionController:
 
         # Old cluster config launch path.
         cluster_config = None
+
         if build_id is None and compute_config is None:
             if session_exists and not config:
                 # Get cluster config stored in DB from existing session
@@ -427,6 +230,9 @@ class SessionController:
 
         # Build and compute template launch path.
         else:
+            # No file mounts are synced when using build id or compute config so can
+            # start standard session using API.
+            start_empty_head_node_only = False
             # Most of the logic is handled in the backend;
             # here, we just need to make sure any CLI options are compatible.
             if min_workers is not None or max_workers is not None:
@@ -478,6 +284,7 @@ class SessionController:
                         restart_only=restart_only,
                         no_restart=no_restart,
                         start_empty_head_node_only=start_empty_head_node_only,
+                        dangerously_set_build_id=dangerously_set_build_id,
                     )
                 ).result
             session_id = up_response.session_id
@@ -566,7 +373,9 @@ class SessionController:
         project_id = get_project_id(project_definition.root)
         session = get_project_session(project_id, session_name, self.api_client)
 
-        cluster_config = get_cluster_config(session_name, self.api_client)
+        cluster_config = get_cluster_config(
+            session_name, self.api_client, disable_project_sync=True
+        )
 
         with format_api_exception(ApiException):
             head_ip = self.api_client.get_session_head_ip_api_v2_sessions_session_id_head_ip_get(
@@ -775,7 +584,7 @@ class SessionController:
                 session_name=session_name,
                 config=None,
                 build_id=None,
-                compute_template_id=None,
+                compute_config=None,
                 min_workers=None,
                 max_workers=None,
                 no_restart=False,
@@ -784,7 +593,6 @@ class SessionController:
                 cloud_id=session.cloud_id,
                 cloud_name=None,
                 idle_timeout=None,
-                yes=True,
             )
         url = get_endpoint(f"/projects/{project_id}/sessions/{session.id}")
         print(f"Pushed to session {session_name}. View at {url}")
