@@ -6,13 +6,26 @@ import click
 
 import anyscale
 
+CREDENTIALS_FILE = "~/.anyscale/credentials.json"
+
+
+def _validate_credential(token: str) -> None:
+    if not token.startswith("sss_"):
+        raise click.ClickException(
+            f"Malformed credentials: {token}\n"
+            f"Please go to {anyscale.conf.ANYSCALE_HOST}/credentials and follow  "
+            "the instructions to properly set the token."
+        )
+
 
 def load_credentials() -> str:
     # The environment variable ANYSCALE_CLI_TOKEN can be used to
     # overwrite the credentials in ~/.anyscale/credentials.json
-    if "ANYSCALE_CLI_TOKEN" in os.environ:
-        return os.environ["ANYSCALE_CLI_TOKEN"]
-    path = os.path.expanduser("~/.anyscale/credentials.json")
+    env_token = os.environ.get("ANYSCALE_CLI_TOKEN")
+    if env_token is not None:
+        _validate_credential(env_token)
+        return env_token
+    path = os.path.expanduser(CREDENTIALS_FILE)
     if not os.path.exists(path):
         raise click.ClickException(
             "Credentials not found. You need to create an account at {0} "
@@ -31,6 +44,8 @@ def load_credentials() -> str:
                 "please contact Anyscale support!".format(anyscale.conf.ANYSCALE_HOST)
             )
             raise click.ClickException(msg)
-    if "cli_token" not in credentials:
+    received_token = credentials.get("cli_token")
+    if received_token is None:
         raise click.ClickException("Credential file not valid, please regenerate it.")
-    return credentials["cli_token"]
+    _validate_credential(received_token)
+    return received_token
