@@ -214,14 +214,16 @@ def get_proj_name_from_id(project_id: str, api_client: DefaultApi) -> str:
 
 
 def get_proj_id_from_name(
-    project_name: str, api_client: Optional[DefaultApi] = None
+    project_name: str,
+    api_client: Optional[DefaultApi] = None,
+    owner: Optional[str] = None,  # this can be the email or the username of the owner
 ) -> str:
     if api_client is None:
         api_client = get_api_client()
 
     with format_api_exception(ApiException):
         resp = api_client.find_project_by_project_name_api_v2_projects_find_by_name_get(
-            name=project_name, _request_timeout=30
+            name=project_name, _request_timeout=30, owner=owner
         )
 
     if not resp.results:
@@ -230,8 +232,31 @@ def get_proj_id_from_name(
             "View the registered projects with `anyscale list projects`."
         )
 
-    project = resp.results[0]
-    return str(project.id)
+    projects = resp.results
+    my_projects = [x for x in projects if x.is_owner]
+
+    selected_project = None
+
+    # If there is more than one result, choose the one that you own
+    # If there is one project, select it
+    if len(projects) == 1:
+        selected_project = projects[0]
+
+    # Only one of the projects is mine. Let's select this one
+    elif len(my_projects) == 1:
+        selected_project = my_projects[0]
+
+    # We know there is at least one element. If none of the projects are mine
+    # then we don't know which one to select
+    else:
+        raise ClickException(
+            f"There are multiple projects '{project_name}' registered with Anyscale. "
+            "View the registered projects with `anyscale list projects`."
+            "Please specify the --owner flag to specify an alternate owner"
+        )
+
+    # Return the id of this project
+    return str(selected_project.id)
 
 
 def clone_cluster_config(
