@@ -23,7 +23,6 @@ from anyscale.commands.session_commands import (
     anyscale_ssh,
     anyscale_stop,
     anyscale_up,
-    anyscale_up_internal,
 )
 from anyscale.commands.session_commands_hidden import (
     anyscale_start,
@@ -31,7 +30,6 @@ from anyscale.commands.session_commands_hidden import (
 )
 from anyscale.commands.snapshot_commands_hidden import snapshot_cli
 import anyscale.conf
-from anyscale.util import send_json_request
 
 
 logger = logging.getLogger(__file__)
@@ -132,39 +130,6 @@ def anyscale_help(ctx: Any) -> None:
     print(ctx.parent.get_help())
 
 
-@click.command(
-    name="cloudgateway",
-    help="Run private clusters via anyscale cloud gateway.",
-    hidden=True,
-)
-@click.option("--gateway-id", type=str, required=True)
-def anyscale_cloudgateway(gateway_id: str) -> None:
-    from anyscale.cloudgateway import CloudGatewayRunner
-
-    # Make sure only registered users can start the gateway.
-    logger.info("Verifying user ...")
-    try:
-        send_json_request("/api/v2/userinfo/", {})
-    except Exception:
-        raise click.ClickException(
-            "Invalid user. Did you set up the cli_token credentials?"
-            + ' To setup your credentials, follow the instructions in the "credentials" tab'
-            + " after logging in to your anyscale account."
-        )
-    anyscale_address = f"/api/v2/cloudgateway/{gateway_id}"
-    cloudgateway_runner = CloudGatewayRunner(anyscale_address)
-    logger.info(
-        "Your gateway-id is: {}. Store it in the provider section in the".format(
-            gateway_id
-        )
-        + " cluster yaml file of the remote cluster that interacts with this gateway."
-        + ' E.g., config["provider"]["gateway_id"]={gateway_id}.'.format(
-            gateway_id=gateway_id
-        )
-    )
-    cloudgateway_runner.gateway_run_forever()
-
-
 cli.add_command(session_cli)
 cli.add_command(snapshot_cli)
 cli.add_command(cloud_cli)
@@ -174,9 +139,7 @@ cli.add_command(list_cli)
 cli.add_command(anyscale_init)
 cli.add_command(anyscale_start)
 cli.add_command(anyscale_up)
-cli.add_command(anyscale_up_internal)
 cli.add_command(anyscale_stop)
-cli.add_command(anyscale_cloudgateway)
 cli.add_command(anyscale_autosync)
 cli.add_command(anyscale_autopush)
 cli.add_command(anyscale_fork)
