@@ -449,18 +449,6 @@ def wait_for_head_node_start(
         retry_count += 1
 
 
-def install_anyscale_hooks(cluster_config: Dict[str, Any]) -> None:
-    "Add anyscale hooks into the cluster configuration."
-
-    cluster_config.setdefault("setup_commands", []).insert(
-        0,
-        # The grep line makes sure that the new line
-        # is not added again to the ~/.bashrc if it is already there.
-        'grep -qF -- "export RAY_ADDRESS=localhost:6379" ~/.bashrc '
-        '|| echo "export RAY_ADDRESS=localhost:6379" >> ~/.bashrc',
-    )
-
-
 def download_anyscale_wheel(api_client: DefaultApi, session_id: str) -> None:
     with format_api_exception(ApiException):
         wheel_resp = api_client.session_get_anyscale_wheel_api_v2_sessions_session_id_anyscale_wheel_get(
@@ -632,4 +620,6 @@ def format_api_exception(exception_type: Type[Exception]) -> Iterator[None]:
         if isinstance(e, ApiException):
             raise click.ClickException(f"{e.status}: {str(e)}")
 
-        raise click.ClickException("{0}: {1}".format(e.status, json.loads(e.body)["error"]["detail"]))  # type: ignore
+        raise click.ClickException(
+            "{0}: {1}".format(e.status, json.loads(e.body)["error"]["detail"])  # type: ignore
+        )
