@@ -39,6 +39,11 @@ class CommandPersister:
                 continue
             cmd = self.queue.pop(0)
             try:
+                if cmd.exec_command:
+                    # Mark exec commands as running by default so that we do not try to create this command on the
+                    # backend ( it is already created).
+                    # When the exec command is finished, command_persister will treat it like other WT commands.
+                    self.running_commands[cmd.scid] = cmd
                 if cmd.scid not in self.running_commands:
                     self.api_client.create_session_command_api_v2_sessions_session_id_create_session_command_post(
                         session_id=self.session_id,
@@ -48,6 +53,7 @@ class CommandPersister:
                             "created_at": cmd.created_at,
                             "finished_at": cmd.finished_at,
                             "status_code": cmd.status_code,
+                            "web_terminal_tab_id": cmd.term_name,
                             "type": SessionCommandTypes.WEBTERMINAL,
                         },
                     )
@@ -81,7 +87,16 @@ class CommandPersister:
                 await asyncio.sleep(1)
 
     def enqueue_command(self, cmd: Command) -> None:
-        self.queue.append(cmd)
+        """
+            enqueue_command adds a Command object to be persisted to our backend.
+            Only enqueue the command to be peristed if it is
+                - just created (therefore not in self.running_commands)
+                  we need to create the entry on the backend.
+                - finished
+                  we need to update the entry on the backend.
+        """
+        if cmd.scid not in self.running_commands or cmd.finished:
+            self.queue.append(cmd)
 
     def kill_running_commands_for_terminal(self, term_name: str) -> None:
         """
