@@ -1,5 +1,6 @@
 import asyncio
-from typing import Any, cast, Dict, Set, Tuple
+import json
+from typing import Any, cast, Dict, Optional, Set, Tuple
 
 import aioredis
 from terminado import NamedTermManager, TermSocket
@@ -9,6 +10,7 @@ from anyscale.webterminal.command_persister import CommandPersister
 from anyscale.webterminal.utils import (
     EXEC_ZSH_CMD,
     extract_commands,
+    generate_id,
     log_commands,
     write_zshrc,
 )
@@ -29,11 +31,24 @@ class AnyscaleTermManager(NamedTermManager):  # type: ignore
         # (keyed by terminal name).
         self.command_persister = command_persister
         self.is_debugger: Dict[str, bool] = {}
+        self.is_exec_command: Dict[str, Optional[str]] = {}
         self.terminal_state: Dict[str, TermState] = {}
 
-    def get_terminal(self, term_name: str, is_debugger: bool = False) -> Any:
+    def clean_up_terminal(self, term_name: str) -> None:
+        del self.terminals[term_name]
+        del self.terminal_state[term_name]
+        del self.is_debugger[term_name]
+        del self.is_exec_command[term_name]
+
+    def get_terminal(
+        self,
+        term_name: str,
+        is_debugger: bool = False,
+        exec_command_id: Optional[str] = None,
+    ) -> Any:
         terminal = super(AnyscaleTermManager, self).get_terminal(term_name)
         self.is_debugger[term_name] = is_debugger
+        self.is_exec_command[term_name] = exec_command_id
         if term_name not in self.terminal_state:
             # We're requesting a new terminal
             self.terminal_state[term_name] = TermState()
@@ -42,6 +57,7 @@ class AnyscaleTermManager(NamedTermManager):  # type: ignore
     def new_named_terminal(self, **kwargs: Any) -> Tuple[str, Any]:
         name, term = super(AnyscaleTermManager, self).new_named_terminal(**kwargs)
         self.is_debugger[name] = False
+        self.is_exec_command[name] = None
         self.terminal_state[name] = TermState()
         term.ptyproc.write(f"{EXEC_ZSH_CMD}\n")
         return name, term
@@ -84,6 +100,20 @@ class AnyscaleTermManager(NamedTermManager):  # type: ignore
                 term_state.curr_cmd = (
                     last_command if not last_command.finished else None
                 )
+                if self.is_exec_command[ptywclients.term_name]:
+                    # Override the id of the command to use the id the backend created
+                    # We only do this for exec command because exec command returns
+                    # the command id immediately to the client.
+                    last_command.scid = cast(
+                        str, self.is_exec_command[ptywclients.term_name]
+                    )
+                    last_command.exec_command = True
+                    if last_command.finished:
+                        # If the terminal is a single command terminal and the command is finished
+                        # we can clean up the terminal.
+                        self.terminate(ptywclients.term_name, force=True)
+                        self.clean_up_terminal(ptywclients.term_name)
+            # Mark the command as either WT or SDK
             log_commands(commands)
             for command in commands:
                 self.command_persister.enqueue_command(command)
@@ -122,7 +152,7 @@ class TerminalPageHandler(tornado.web.RequestHandler):
         command_persister = self.application.settings["command_persister"]
         term_manager.terminate(term_name, force=True)
         command_persister.kill_running_commands_for_terminal(term_name)
-        del term_manager.terminals[term_name]
+        term_manager.clean_up_terminal(term_name)
 
 
 class TerminalListHandler(tornado.web.RequestHandler):
@@ -130,11 +160,19 @@ class TerminalListHandler(tornado.web.RequestHandler):
 
     def get(self) -> None:
         term_manager = self.application.settings["term_manager"]
-
+        include_all = json.loads(
+            self.get_query_argument("all_terms", default="false")
+            or "false"  # this additional false is added bc mypy complains.
+        )
         data = {
             "terminals": [
-                {"id": term_name, "debugger": term_manager.is_debugger[term_name]}
+                {
+                    "id": term_name,
+                    "debugger": term_manager.is_debugger[term_name],
+                    "exec_command": term_manager.is_exec_command[term_name] is not None,
+                }
                 for term_name in term_manager.terminals.keys()
+                if (term_manager.is_exec_command[term_name] is None or include_all)
             ]
         }
 
@@ -149,12 +187,55 @@ class NewTerminalHandler(tornado.web.RequestHandler):
         self.write({"id": term_name})
 
 
-class TerminalCommandHandler(tornado.web.RequestHandler):
-    """Used to kill terminal commands"""
+class ExecTerminalHandler(tornado.web.RequestHandler):
+    """Create new unused terminal"""
 
-    def delete(self, session_command_id: str) -> None:
-        # TODO
-        return None
+    def prepare(self) -> None:
+        # Incorporate request JSON into arguments dictionary.
+        if self.request.body:
+            try:
+                json_data = json.loads(self.request.body)
+                self.request.body_arguments = json_data
+            except ValueError:
+                message = "Unable to parse JSON."
+                self.send_error(400, message=message)  # Bad Request
+
+    async def post(self) -> None:
+        """
+          POST /exec/ accepts { shell_command, command_id }
+            - shell_command, the command to run
+            - command_id, the id of the session_command entry created on the backend
+
+            returns: { web_terminal_id }
+            this is the web_terminal in which this command was run in.
+            We can use this to kill the command.
+        """
+        # if default is not specified, the argument is considered required and an exeception will
+        # be raised by tornado.
+        shell_command = self.request.body_arguments.get("shell_command")
+        command_id = self.request.body_arguments.get("command_id")
+        term_manager = self.application.settings["term_manager"]
+        term_id = generate_id()
+        term = term_manager.get_terminal(
+            term_id, is_debugger=False, exec_command_id=command_id
+        )
+        term.ptyproc.write(f"{EXEC_ZSH_CMD}\n")
+        await asyncio.sleep(0.5)
+        # Wait for zsh to load
+        while True:
+            term_state = term_manager.terminal_state.get(term_id, None)
+            if term_state and term_state.has_launched_zsh:
+                precmd_zsh_output = "".join(term.preopen_buffer)
+                # zsh complains about the directories used in the CI.
+                # We need to enter n so it doesn't do a security check.
+                if "zsh compinit: insecure directories" in precmd_zsh_output:
+                    term.ptyproc.write("n\n")
+                break
+            await asyncio.sleep(0.5)
+
+        # Write command
+        term.ptyproc.write(f"{shell_command}\n")
+        self.write({"web_terminal_tab_id": term_id})
 
 
 def make_application(
@@ -175,8 +256,8 @@ def make_application(
         ),
         (r"/webterminal/new/?", NewTerminalHandler),
         (r"/webterminal/list", TerminalListHandler),
+        (r"/webterminal/exec", ExecTerminalHandler),
         (r"/webterminal/(\w+)/?", TerminalPageHandler),
-        (r"/webterminal/command/(\w+)/?", TerminalCommandHandler),
     ]
     return tornado.web.Application(
         handlers,
