@@ -45,9 +45,13 @@ def test_setup_cloud_aws(mock_api_client: Mock) -> None:
         CloudController, "setup_aws", return_value=None
     ) as mock_setup_aws:
         cloud_controller = CloudController(api_client=mock_api_client)
-        cloud_controller.setup_cloud(provider="aws", region=None, name="test-aws")
+        cloud_controller.setup_cloud(
+            provider="aws", region=None, name="test-aws", yes=False
+        )
 
-        mock_setup_aws.assert_called_once_with(region="us-west-2", name="test-aws")
+        mock_setup_aws.assert_called_once_with(
+            region="us-west-2", name="test-aws", yes=False
+        )
 
 
 def test_setup_cloud_gcp(mock_api_client: Mock) -> None:
@@ -68,7 +72,10 @@ def test_setup_cloud_invalid_provider(mock_api_client: Mock) -> None:
     cloud_controller = CloudController(api_client=mock_api_client)
     with pytest.raises(ClickException):
         cloud_controller.setup_cloud(
-            provider="azure", region="azure-west-1", name="invalid cloud provider"
+            provider="azure",
+            region="azure-west-1",
+            name="invalid cloud provider",
+            yes=False,
         )
 
 
