@@ -1,4 +1,5 @@
 import os
+from typing import Optional
 from unittest.mock import ANY, Mock, patch
 
 import pytest
@@ -29,7 +30,10 @@ def mock_api_client(project_test_data: Project) -> Mock:
     return mock_api_client
 
 
-def test_clone_project(project_test_data: Project, mock_api_client: Mock) -> None:
+@pytest.mark.parametrize("owner", [None, "owner"])
+def test_clone_project(
+    project_test_data: Project, mock_api_client: Mock, owner: Optional[str]
+) -> None:
     project_controller = ProjectController(api_client=mock_api_client)
     project_controller._write_sample_compute_config = Mock()  # type: ignore
 
@@ -37,10 +41,10 @@ def test_clone_project(project_test_data: Project, mock_api_client: Mock) -> Non
     with patch.multiple("os", makedirs=os_makedirs_mock), patch(
         "anyscale.project._write_cluster_config_to_disk"
     ) as write_cluster_config_mock:
-        project_controller.clone(project_name=project_test_data.name)
+        project_controller.clone(project_name=project_test_data.name, owner=owner)
 
     mock_api_client.find_project_by_project_name_api_v2_projects_find_by_name_get.assert_called_once_with(
-        name=project_test_data.name, _request_timeout=ANY,
+        name=project_test_data.name, _request_timeout=ANY, owner=owner
     )
     mock_api_client.get_project_latest_cluster_config_api_v2_projects_project_id_latest_cluster_config_get.assert_called_once_with(
         project_test_data.id,
