@@ -3,7 +3,7 @@ import json
 import os
 import tempfile
 from typing import Iterator, Optional, Tuple
-from unittest.mock import ANY, call, Mock, mock_open, patch
+from unittest.mock import ANY, call, MagicMock, Mock, mock_open, patch
 
 import click
 import pytest
@@ -11,24 +11,58 @@ import yaml
 
 from anyscale.client.openapi_client import (  # type: ignore
     Cloud,
+    ClusterConfig,
     CreateSessionFromSnapshotOptions,
     Project,
     ProjectDefaultSessionName,
     ProjectdefaultsessionnameResponse,
     ProjectListResponse,
     Session,
-    SessionFinishUpOptions,
     SessionListResponse,
     SessionResponse,
     SessionUpOptions,
-    SessionUpResponse,
-    SessionupresponseResponse,
+    SetupInitializeSessionOptions,
+    StartEmptySessionResponse,
+    StartemptysessionresponseResponse,
     StopSessionOptions,
 )
 from anyscale.controllers.session_controller import SessionController
 from anyscale.project import CLUSTER_YAML_TEMPLATE
 
 
+COMPUTE_CONFIG_TEMPLATE = json.dumps(
+    {
+        "cloud_id": "fake-cloud-id",
+        "region": "fake-region",
+        "allowed_azs": ["fake-az1"],
+        "head_node_type": {
+            "name": "head-node-name",
+            "instance_type": "fake-head-instance-type",
+        },
+        "worker_node_types": [
+            {
+                "name": "worker-node-name",
+                "instance_type": "fake-worker-instance-type",
+                "min_workers": 0,
+                "max_workers": 10,
+                "use_spot": True,
+            }
+        ],
+        "aws": {
+            "SubnetId": "fake-subnet-id",
+            "SecurityGroupIds": ["fake-security-group-id"],
+            "IamInstanceProfile": "fake-iam-arn",
+            "TagSpecifications": [
+                {
+                    "ResourceType": "instance",
+                    "Tags": [{"Key": "fake-key", "Value": "fake-value"}],
+                },
+            ],
+        },
+    }
+)
+
+
 @pytest.fixture()
 def mock_api_client(mock_api_client_with_session: Mock) -> Mock:
     mock_api_client = mock_api_client_with_session
@@ -54,17 +88,13 @@ def test_stop_single_session(session_test_data: Session, mock_api_client: Mock)
         get_project_id=mock_get_project_id,
     ):
         session_controller.stop(
-            "test-name",
-            terminate=False,
-            delete=False,
-            workers_only=False,
-            keep_min_workers=False,
+            "test-name", delete=False, workers_only=False, keep_min_workers=False,
         )
 
     mock_api_client.stop_session_api_v2_sessions_session_id_stop_post.assert_called_once_with(
         session_test_data.id,
         StopSessionOptions(
-            terminate=False, delete=False, workers_only=False, keep_min_workers=False
+            terminate=True, delete=False, workers_only=False, keep_min_workers=False
         ),
     )
 
@@ -86,6 +116,7 @@ def mock_api_client_multiple_sessions(base_mock_api_client: Mock) -> Mock:
                 state="Running",
                 head_node_ip="127.0.0.1",
                 idle_timeout=120,
+                access_token="value",
             ),
             Session(
                 id="ses_2",
@@ -97,6 +128,7 @@ def mock_api_client_multiple_sessions(base_mock_api_client: Mock) -> Mock:
                 state="Running",
                 head_node_ip="127.0.0.1",
                 idle_timeout=120,
+                access_token="value",
             ),
         ]
     )
@@ -126,11 +158,7 @@ def test_stop_multiple_sessions(mock_api_client_multiple_sessions: Mock) -> None
         get_project_id=mock_get_project_id,
     ):
         session_controller.stop(
-            "ses_?",
-            terminate=True,
-            delete=True,
-            workers_only=True,
-            keep_min_workers=True,
+            "ses_?", delete=True, workers_only=True, keep_min_workers=True,
         )
 
     mock_api_client.stop_session_api_v2_sessions_session_id_stop_post.assert_has_calls(
@@ -172,11 +200,6 @@ def mock_up_tuple(
     )
     mock_project_definition = Mock()
     mock_project_definition.root = "/some/directory"
-    mock_popen_output = Mock()
-    mock_popen_output.stdout = []
-    mock_popen_output.returncode = 0
-    mock_popen_output.communicate = Mock()
-    subprocess_mock = Mock(return_value=mock_popen_output)
     mock_wait_for_session_start = Mock()
 
     with patch.multiple(
@@ -186,11 +209,12 @@ def mock_up_tuple(
         get_cloud_id_and_name=Mock(
             return_value=tuple([cloud_test_data.id, cloud_test_data.name])
         ),
-        download_anyscale_wheel=Mock(return_value=None),
-        configure_for_session=Mock(return_value=yaml.safe_load(CLUSTER_YAML_TEMPLATE),),
-        get_head_node_ip=Mock(return_value="1.2.3.4"),
         wait_for_session_start=mock_wait_for_session_start,
-    ), patch.multiple("subprocess", Popen=subprocess_mock):
+        wait_for_head_node_start=mock_wait_for_session_start,
+        _update_file_mounts=MagicMock(
+            side_effect=lambda cluster_config, project_root_dir, api_client: cluster_config
+        ),
+    ):
         yield base_mock_api_client, project_test_data, cloud_test_data, mock_wait_for_session_start
 
 
@@ -202,6 +226,7 @@ def test_anyscale_up_success(mock_up_tuple: Tuple[Mock, Project, Cloud, Mock]) -
         mock_wait_for_start,
     ) = mock_up_tuple
     session_controller = SessionController(api_client=mock_api_client)
+    session_controller.push = Mock()  # type: ignore
     mock_api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get.return_value = ProjectdefaultsessionnameResponse(
         result=ProjectDefaultSessionName(name="session-11")
     )
@@ -209,14 +234,10 @@ def test_anyscale_up_success(mock_up_tuple: Tuple[Mock, Project, Cloud, Mock]) -
         results=[]  # No session with this name yet, brand new session.
     )
     # Session Name should be "11"
-    mock_api_client.session_up_api_v2_sessions_up_post = Mock(
-        return_value=SessionupresponseResponse(
-            result=SessionUpResponse(
-                session_id="session-10",
-                cluster_name="new_cluster",
-                private_key="key",
-                cluster_config=yaml.safe_load(CLUSTER_YAML_TEMPLATE),
-                is_same_cluster_config=False,
+    mock_api_client.start_empty_session_api_v2_sessions_start_empty_session_post = Mock(
+        return_value=StartemptysessionresponseResponse(
+            result=StartEmptySessionResponse(
+                session_id="session-10", second_update_required=False,
             )
         )
     )
@@ -228,7 +249,6 @@ def test_anyscale_up_success(mock_up_tuple: Tuple[Mock, Project, Cloud, Mock]) -
             session_name=None,
             config=temp_file.name,
             build_id=None,
-            compute_template_id=None,
             min_workers=None,
             max_workers=None,
             no_restart=False,
@@ -237,9 +257,9 @@ def test_anyscale_up_success(mock_up_tuple: Tuple[Mock, Project, Cloud, Mock]) -
             cloud_id=None,
             cloud_name=None,
             idle_timeout=-1,
-            yes=True,
+            compute_config=None,
         )
-        mock_api_client.session_up_api_v2_sessions_up_post.assert_called_with(
+        mock_api_client.start_empty_session_api_v2_sessions_start_empty_session_post.assert_called_once_with(
             session_up_options=SessionUpOptions(
                 project_id=project_test_data.id,
                 name="session-11",
@@ -248,140 +268,21 @@ def test_anyscale_up_success(mock_up_tuple: Tuple[Mock, Project, Cloud, Mock]) -
                 },
                 cloud_id=cloud_test_data.id,
                 idle_timeout=-1,
+                start_empty_head_node_only=True,
             )
         )
 
-    mock_api_client.session_up_api_v2_sessions_up_post.assert_called_once()
-
-
-def test_anyscale_up_templates_success(
-    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock]
-) -> None:
-    (
-        mock_api_client,
-        project_test_data,
-        cloud_test_data,
-        mock_wait_for_start,
-    ) = mock_up_tuple
-
-    session_controller = SessionController(api_client=mock_api_client)
-    mock_api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get.return_value = ProjectdefaultsessionnameResponse(
-        result=ProjectDefaultSessionName(name="session-11")
-    )
-    mock_api_client.list_sessions_api_v2_sessions_get.return_value = SessionListResponse(
-        results=[]  # No session with this name yet, brand new session.
-    )
-
-    mock_api_client.session_up_api_v2_sessions_up_post = Mock(
-        return_value=SessionupresponseResponse(
-            result=SessionUpResponse(
-                session_id="session-10",
-                cluster_name="new_cluster",
-                private_key="key",
-                cluster_config=yaml.safe_load(CLUSTER_YAML_TEMPLATE),
-                is_same_cluster_config=False,
-            )
-        )
-    )
-
-    session_controller.up(
-        session_name=None,
-        config=None,
-        build_id="build-id",
-        compute_template_id="ct-id",
-        min_workers=None,
-        max_workers=None,
-        idle_timeout=-1,
-        no_restart=False,
-        restart_only=False,
-        disable_sync=False,
-        cloud_id=None,
-        cloud_name=None,
-        yes=True,
-    )
-    # Ensure that the API call is made with the
-    # build ID and compute template ID,
-    # and not the cluster config.
-    mock_api_client.session_up_api_v2_sessions_up_post.assert_called_with(
-        session_up_options=SessionUpOptions(
-            project_id=project_test_data.id,
-            name="session-11",
-            cluster_config=None,
-            idle_timeout=-1,
-            build_id="build-id",
-            compute_template_id="ct-id",
-            cloud_id=cloud_test_data.id,
-        )
-    )
-    mock_api_client.session_up_api_v2_sessions_up_post.assert_called_once()
-
-
-def test_anyscale_up_templates_badparams(
-    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock]
-) -> None:
-    (
-        mock_api_client,
-        project_test_data,
-        cloud_test_data,
-        mock_wait_for_start,
-    ) = mock_up_tuple
-
-    session_controller = SessionController(api_client=mock_api_client)
-    mock_api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get.return_value = ProjectdefaultsessionnameResponse(
-        result=ProjectDefaultSessionName(name="session-11")
-    )
-    mock_api_client.list_sessions_api_v2_sessions_get.return_value = SessionListResponse(
-        results=[]  # No session with this name yet, brand new session.
-    )
-
-    mock_api_client.session_up_api_v2_sessions_up_post = Mock(
-        return_value=SessionupresponseResponse(
-            result=SessionUpResponse(
-                session_id="session-10",
-                cluster_name="new_cluster",
-                private_key="key",
-                cluster_config=yaml.safe_load(CLUSTER_YAML_TEMPLATE),
-                is_same_cluster_config=False,
-            )
-        )
-    )
-
-    # Specifying only the build ID or only the compute template ID
-    # should raise a ValueError.
-    # (If neither are specified, it counts as a cluster config
-    # startup path, and is handled in other existing tests.)
-    with pytest.raises(ValueError):
-        session_controller.up(
-            session_name=None,
-            config=None,
-            build_id="build-id",
-            compute_template_id=None,
-            min_workers=None,
-            max_workers=None,
-            idle_timeout=-1,
-            no_restart=False,
-            restart_only=False,
-            disable_sync=False,
-            cloud_id=None,
-            cloud_name=None,
-            yes=True,
-        )
-
-    with pytest.raises(ValueError):
-        session_controller.up(
-            session_name=None,
-            config=None,
-            build_id=None,
-            compute_template_id="ct-id",
-            min_workers=None,
-            max_workers=None,
-            idle_timeout=-1,
-            no_restart=False,
-            restart_only=False,
-            disable_sync=False,
-            cloud_id=None,
-            cloud_name=None,
-            yes=True,
+        mock_api_client.setup_and_initialize_session_api_v2_sessions_session_id_setup_and_initialize_session_post.assert_called_once_with(
+            session_id="session-10",
+            setup_initialize_session_options=SetupInitializeSessionOptions(
+                project_id=project_test_data.id,
+                cluster_config={
+                    "config": json.dumps(yaml.safe_load(CLUSTER_YAML_TEMPLATE))
+                },
+                restart_only=False,
+                no_restart=False,
+                no_update=True,
+            ),
         )
 
 
@@ -393,6 +294,7 @@ def test_anyscale_up_failure(mock_up_tuple: Tuple[Mock, Project, Cloud, Mock]) -
         mock_wait_for_start,
     ) = mock_up_tuple
     session_controller = SessionController(api_client=mock_api_client)
+    session_controller.push = Mock()  # type: ignore
     sesion_id = "session-10"
     mock_api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get.return_value = ProjectdefaultsessionnameResponse(
         result=ProjectDefaultSessionName(name="session-11")
@@ -401,24 +303,15 @@ def test_anyscale_up_failure(mock_up_tuple: Tuple[Mock, Project, Cloud, Mock]) -
         results=[]  # No session with this name yet, brand new session.
     )
     # Session Name should be "11"
-    mock_api_client.session_up_api_v2_sessions_up_post = Mock(
-        return_value=SessionupresponseResponse(
-            result=SessionUpResponse(
-                session_id=sesion_id,
-                cluster_name="new_cluster",
-                private_key="key",
-                cluster_config=yaml.safe_load(CLUSTER_YAML_TEMPLATE),
-                is_same_cluster_config=False,
+    mock_api_client.start_empty_session_api_v2_sessions_start_empty_session_post = Mock(
+        return_value=StartemptysessionresponseResponse(
+            result=StartEmptySessionResponse(
+                session_id=sesion_id, second_update_required=False,
             )
         )
     )
     mock_wait_for_start.side_effect = click.ClickException("Node Failed to Start!")
 
-    # We are setting the mock to return an error code
-    import subprocess
-
-    subprocess.Popen.return_value.returncode = 100  # type: ignore
-
     with tempfile.NamedTemporaryFile(mode="w") as temp_file:
         temp_file.write(CLUSTER_YAML_TEMPLATE)
         temp_file.flush()
@@ -427,7 +320,6 @@ def test_anyscale_up_failure(mock_up_tuple: Tuple[Mock, Project, Cloud, Mock]) -
                 session_name=None,
                 config=temp_file.name,
                 build_id=None,
-                compute_template_id=None,
                 min_workers=None,
                 max_workers=None,
                 no_restart=False,
@@ -436,17 +328,8 @@ def test_anyscale_up_failure(mock_up_tuple: Tuple[Mock, Project, Cloud, Mock]) -
                 cloud_id=None,
                 cloud_name=None,
                 idle_timeout=-1,
-                yes=True,
+                compute_config=None,
             )
-        mock_api_client.session_finish_up_api_v2_sessions_session_id_finish_up_post.assert_called_once_with(
-            session_finish_up_options=SessionFinishUpOptions(
-                startup_log="",
-                new_session=True,
-                head_node_ip="1.2.3.4",
-                cluster_launch_errored=True,
-            ),
-            session_id=sesion_id,
-        )
 
 
 def test_anyscale_up_no_config_provided(
@@ -459,6 +342,7 @@ def test_anyscale_up_no_config_provided(
         mock_wait_for_start,
     ) = mock_up_tuple
     session_controller = SessionController(api_client=mock_api_client)
+    session_controller.push = Mock()  # type: ignore
     mock_api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get.return_value = ProjectdefaultsessionnameResponse(
         result=ProjectDefaultSessionName(name="session-11")
     )
@@ -468,22 +352,22 @@ def test_anyscale_up_no_config_provided(
         ]  # This session already exists so no config needs to be provided
     )
     # Session Name should be "11"
-    mock_api_client.session_up_api_v2_sessions_up_post = Mock(
-        return_value=SessionupresponseResponse(
-            result=SessionUpResponse(
-                session_id="session-10",
-                cluster_name="new_cluster",
-                private_key="key",
-                cluster_config=yaml.safe_load(CLUSTER_YAML_TEMPLATE),
-                is_same_cluster_config=False,
+    mock_api_client.start_empty_session_api_v2_sessions_start_empty_session_post = Mock(
+        return_value=StartemptysessionresponseResponse(
+            result=StartEmptySessionResponse(
+                session_id="session-10", second_update_required=False,
             )
         )
     )
+    mock_api_client.get_session_cluster_config_api_v2_sessions_session_id_cluster_config_get.return_value = Mock(
+        result=ClusterConfig(
+            config=CLUSTER_YAML_TEMPLATE, config_with_defaults=CLUSTER_YAML_TEMPLATE
+        )
+    )
     session_controller.up(
         session_name=None,
         config=None,
         build_id=None,
-        compute_template_id=None,
         min_workers=None,
         max_workers=None,
         no_restart=False,
@@ -492,22 +376,36 @@ def test_anyscale_up_no_config_provided(
         cloud_id=None,
         cloud_name=None,
         idle_timeout=-1,
-        yes=True,
+        compute_config=None,
     )
-    mock_api_client.session_up_api_v2_sessions_up_post.assert_called_with(
+    mock_api_client.start_empty_session_api_v2_sessions_start_empty_session_post.assert_called_once_with(
         session_up_options=SessionUpOptions(
             project_id=project_test_data.id,
             name="session-11",
-            cluster_config=None,
+            cluster_config={
+                "config": json.dumps(yaml.safe_load(CLUSTER_YAML_TEMPLATE))
+            },
             cloud_id=cloud_test_data.id,
             idle_timeout=-1,
+            start_empty_head_node_only=True,
         )
     )
 
-    mock_api_client.session_up_api_v2_sessions_up_post.assert_called_once()
+    mock_api_client.setup_and_initialize_session_api_v2_sessions_session_id_setup_and_initialize_session_post.assert_called_once_with(
+        session_id="session-10",
+        setup_initialize_session_options=SetupInitializeSessionOptions(
+            project_id=project_test_data.id,
+            cluster_config={
+                "config": json.dumps(yaml.safe_load(CLUSTER_YAML_TEMPLATE))
+            },
+            restart_only=False,
+            no_restart=False,
+            no_update=True,
+        ),
+    )
 
 
-def test_anyscale_up_config_provided(
+def test_anyscale_app_template(
     mock_up_tuple: Tuple[Mock, Project, Cloud, Mock]
 ) -> None:
     (
@@ -517,35 +415,32 @@ def test_anyscale_up_config_provided(
         mock_wait_for_start,
     ) = mock_up_tuple
     session_controller = SessionController(api_client=mock_api_client)
+    session_controller.push = Mock()  # type: ignore
     mock_api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get.return_value = ProjectdefaultsessionnameResponse(
         result=ProjectDefaultSessionName(name="session-11")
     )
     mock_api_client.list_sessions_api_v2_sessions_get.return_value = SessionListResponse(
-        results=[
-            Session
-        ]  # This session already exists so no config needs to be provided
+        results=[]  # No session with this name yet, brand new session.
     )
     # Session Name should be "11"
-    mock_api_client.session_up_api_v2_sessions_up_post = Mock(
-        return_value=SessionupresponseResponse(
-            result=SessionUpResponse(
-                session_id="session-10",
-                cluster_name="new_cluster",
-                private_key="key",
-                cluster_config=yaml.safe_load(CLUSTER_YAML_TEMPLATE),
-                is_same_cluster_config=False,
+    mock_api_client.start_empty_session_api_v2_sessions_start_empty_session_post = Mock(
+        return_value=StartemptysessionresponseResponse(
+            result=StartEmptySessionResponse(
+                session_id="session-10", second_update_required=False,
             )
         )
     )
+    mock_api_client.create_compute_template_api_v2_compute_templates_post = Mock(
+        return_value=Mock(result=Mock(id="mock_compute_template_id"))
+    )
     with tempfile.NamedTemporaryFile(mode="w") as temp_file:
-        temp_file.write(CLUSTER_YAML_TEMPLATE)
+        temp_file.write(COMPUTE_CONFIG_TEMPLATE)
         temp_file.flush()
 
         session_controller.up(
             session_name=None,
-            config=temp_file.name,
-            build_id=None,
-            compute_template_id=None,
+            config=None,
+            build_id="mock_build_id",
             min_workers=None,
             max_workers=None,
             no_restart=False,
@@ -554,21 +449,31 @@ def test_anyscale_up_config_provided(
             cloud_id=None,
             cloud_name=None,
             idle_timeout=-1,
-            yes=True,
+            compute_config=temp_file.name,
         )
-        mock_api_client.session_up_api_v2_sessions_up_post.assert_called_with(
+        mock_api_client.start_empty_session_api_v2_sessions_start_empty_session_post.assert_called_once_with(
             session_up_options=SessionUpOptions(
                 project_id=project_test_data.id,
                 name="session-11",
-                cluster_config={
-                    "config": json.dumps(yaml.safe_load(CLUSTER_YAML_TEMPLATE))
-                },
+                cluster_config=None,
+                build_id="mock_build_id",
+                compute_template_id="mock_compute_template_id",
                 cloud_id=cloud_test_data.id,
                 idle_timeout=-1,
+                start_empty_head_node_only=False,
             )
         )
 
-    mock_api_client.session_up_api_v2_sessions_up_post.assert_called_once()
+        mock_api_client.setup_and_initialize_session_api_v2_sessions_session_id_setup_and_initialize_session_post.assert_called_once_with(
+            session_id="session-10",
+            setup_initialize_session_options=SetupInitializeSessionOptions(
+                project_id=project_test_data.id,
+                cluster_config={"config": "null"},
+                restart_only=False,
+                no_restart=False,
+                no_update=True,
+            ),
+        )
 
 
 def test_ssh_session(
@@ -602,7 +507,9 @@ def test_ssh_session(
     mock_load_project_or_throw.assert_called_once_with()
     mock_get_project_id.assert_called_once_with("/some/directory")
     mock_get_cluster_config.assert_called_once_with(
-        session_test_data.name, mock_api_client_multiple_sessions
+        session_test_data.name,
+        mock_api_client_multiple_sessions,
+        disable_project_sync=True,
     )
 
 
@@ -848,7 +755,7 @@ def test_anyscale_push_session(
             session_name=session_test_data.name,
             config=None,
             build_id=None,
-            compute_template_id=None,
+            compute_config=None,
             min_workers=None,
             max_workers=None,
             no_restart=False,
@@ -857,7 +764,6 @@ def test_anyscale_push_session(
             disable_sync=True,
             cloud_id=session_test_data.cloud_id,
             cloud_name=None,
-            yes=True,
         )
 
 
