@@ -99,3 +99,40 @@ def test_update_file_mounts_with_working_dir(config_patches: str) -> None:
     ] = working_dir
     new_config = _update_file_mounts(input_config, project_root_dir)
     assert new_config["file_mounts"] == {working_dir: project_root_dir}
+
+
+def test_update_file_mounts_if_proj_dir_exists(config_patches: str) -> None:
+    # Updates project directory sync in file mounts when there is already a file
+    # mount in the cluster config that maps to the remote project directory location.
+    project_usr1_root_dir = "/home/usr1/root/dir"
+    project_usr2_root_dir = "/home/usr2/root/dir"
+    remote_project_directory_name = f"/home/ray/{config_patches}"
+
+    mock_path_exists = Mock(
+        side_effect=lambda x: True if x == project_usr2_root_dir else False
+    )
+    mock_path_samefile = Mock(side_effect=lambda x, y: x == y)
+
+    with patch.multiple(
+        "os.path", exists=mock_path_exists, samefile=mock_path_samefile
+    ):
+        # Handles the case when the local directory in the project directory sync doesn't exist. This can
+        # happen when getting the cluster config on a session that is shared, or if the local directory
+        # no longer exists.
+        input_config: Dict[str, Any] = {
+            "file_mounts": {remote_project_directory_name: project_usr1_root_dir}
+        }
+        new_config = _update_file_mounts(input_config, project_usr2_root_dir)
+        assert new_config["file_mounts"] == {
+            remote_project_directory_name: project_usr2_root_dir
+        }
+
+        # Handles the case when the project directory sync is already correctlly updated. This happens for
+        # subsequent commands on a session you created.
+        input_config = {
+            "file_mounts": {remote_project_directory_name: project_usr2_root_dir}
+        }
+        new_config = _update_file_mounts(input_config, project_usr2_root_dir)
+        assert new_config["file_mounts"] == {
+            remote_project_directory_name: project_usr2_root_dir
+        }
