@@ -8,11 +8,12 @@ import pytest
 import yaml
 
 import anyscale
-from anyscale.client.openapi_client.models.application_template import ApplicationTemplate  # type: ignore
+from anyscale.client.openapi_client.models.app_config import AppConfig  # type: ignore
 from anyscale.client.openapi_client.models.build import Build  # type: ignore
 from anyscale.client.openapi_client.models.project import Project  # type: ignore
 from anyscale.client.openapi_client.models.project_response import ProjectResponse  # type: ignore
 from anyscale.client.openapi_client.models.session import Session  # type: ignore
+from anyscale.connect import _get_wheel_url
 from anyscale.connect import SessionBuilder
 
 
@@ -32,11 +33,12 @@ def _make_session(i: int, state: str) -> Session:
         jupyter_notebook_url="http://session-{}.userdata.com/jupyter/lab?token=value".format(
             i
         ),
+        access_token="value",
     )
 
 
-def _make_app_template() -> ApplicationTemplate:
-    return ApplicationTemplate(
+def _make_app_template() -> AppConfig:
+    return AppConfig(
         project_id="project_id",
         id="application_template_id",
         name="test-app-config",
@@ -61,7 +63,9 @@ def _make_build() -> Build:
 
 
 def _make_test_builder(
-    tmp_path: Path, session_states: List[str] = ["Running"]
+    tmp_path: Path,
+    session_states: List[str] = ["Running"],
+    setup_project_dir: bool = True,
 ) -> Tuple[Any, Any, Any, Any]:
     scratch = tmp_path / "scratch"
     sdk = Mock()
@@ -85,9 +89,14 @@ def _make_test_builder(
         scratch_dir=scratch.absolute().as_posix(),
         anyscale_sdk=sdk,
         subprocess=subprocess,
-        ray=ray,
+        _ray=ray,
         _os=_os,
+        _ignore_version_check=True,
     )
+    if setup_project_dir:
+        builder.project_dir(scratch.absolute().as_posix())
+    else:
+        builder._in_shell = lambda: True
     builder._find_project_id = lambda _: None  # type: ignore
     setattr(builder, "_up_session", Mock())
     setattr(
@@ -106,7 +115,11 @@ def test_new_proj_connect_params(tmp_path: Path, project_test_data: Project) ->
 
     assert anyscale.project.get_project_id(project_dir)
     builder._up_session.assert_called_once_with(
-        ANY, "session-0", "anyscale_default_cloud", project_dir,
+        ANY,
+        "session-0",
+        "anyscale_default_cloud",
+        project_dir,
+        dangerously_set_build_id=None,
     )
 
     # Also check connection params in this test.
@@ -122,7 +135,9 @@ def test_detect_existing_proj(tmp_path: Path) -> None:
     nested_dir = (tmp_path / "my_proj" / "nested").absolute().as_posix()
     parent_dir = os.path.dirname(nested_dir)
     os.makedirs(nested_dir)
-    builder, sdk, subprocess, ray = _make_test_builder(tmp_path)
+    builder, sdk, subprocess, ray = _make_test_builder(
+        tmp_path, setup_project_dir=False
+    )
 
     # Setup project in parent dir
     project_yaml = os.path.join(parent_dir, ".anyscale.yaml")
@@ -138,7 +153,11 @@ def test_detect_existing_proj(tmp_path: Path) -> None:
         os.chdir(cwd)
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-0", "anyscale_default_cloud", parent_dir,
+        ANY,
+        "session-0",
+        "anyscale_default_cloud",
+        parent_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -152,7 +171,11 @@ def test_fallback_scratch_dir(tmp_path: Path, project_test_data: Project) -> Non
 
     assert anyscale.project.get_project_id(scratch_dir)
     builder._up_session.assert_called_once_with(
-        ANY, "session-0", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-0",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -209,13 +232,15 @@ def test_connect_with_cloud(tmp_path: Path, project_test_data: Project) -> None:
     builder.cloud("test_cloud").connect()
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-0", "test_cloud", scratch_dir,
+        ANY, "session-0", "test_cloud", scratch_dir, dangerously_set_build_id=None
     )
 
 
 def test_clone_scratch_dir(tmp_path: Path, project_test_data: Project) -> None:
     scratch_dir = (tmp_path / "scratch").absolute().as_posix()
-    builder, sdk, subprocess, ray = _make_test_builder(tmp_path)
+    builder, sdk, subprocess, ray = _make_test_builder(
+        tmp_path, setup_project_dir=False
+    )
     builder._find_project_id = lambda _: "foo"
     sdk.create_project.return_value = ProjectResponse(result=project_test_data)
 
@@ -234,7 +259,11 @@ def test_clone_scratch_dir(tmp_path: Path, project_test_data: Project) -> None:
         ["anyscale", "clone", "scratch"]
     )
     builder._up_session.assert_called_once_with(
-        ANY, "session-0", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-0",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -255,7 +284,11 @@ def test_new_session(tmp_path: Path, project_test_data: Project) -> None:
     builder.connect()
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-0", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-0",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -357,7 +390,11 @@ def test_new_session_lost_lock(tmp_path: Path, project_test_data: Project) -> No
         builder.connect()
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-0", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-0",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -372,7 +409,7 @@ def test_reuse_session_hash_match(tmp_path: Path, project_test_data: Project) ->
     os.makedirs(scratch_dir)
     builder.require(["wikipedia", "dask"]).project_dir(scratch_dir)
     cluster_yaml = yaml.safe_load(anyscale.project.CLUSTER_YAML_TEMPLATE)
-    builder._populate_cluster_config(cluster_yaml, "project_id", "scratch")
+    builder._populate_cluster_config(cluster_yaml, "project_id", "scratch", build=None)
     with open(os.path.join(scratch_dir, "session-default.yaml"), "w+") as f:
         f.write(yaml.dump(cluster_yaml))
     local_files_hash = builder._fingerprint(scratch_dir).encode("utf-8")
@@ -392,7 +429,11 @@ def test_reuse_session_hash_match(tmp_path: Path, project_test_data: Project) ->
     builder.require(["wikipedia", "dask", "celery"]).connect()
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-0", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-0",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -417,7 +458,11 @@ def test_reuse_session_hash_mismatch(
     builder.connect()
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-0", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-0",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -452,7 +497,11 @@ def test_reuse_session_lock_failure(tmp_path: Path, project_test_data: Project)
     builder.connect()
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-1", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-1",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -478,7 +527,11 @@ def test_restart_session_conn_failure(
         builder.connect()
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-0", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-0",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -504,7 +557,11 @@ def test_skip_session_conn_failure(tmp_path: Path, project_test_data: Project) -
     builder.connect()
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-1", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-1",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -519,7 +576,11 @@ def test_fixed_session(tmp_path: Path, project_test_data: Project) -> None:
     builder.session("session-1", update=True).connect()
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-1", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-1",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -534,7 +595,11 @@ def test_fixed_session_not_running(tmp_path: Path, project_test_data: Project) -
     builder.session("session-1").connect()
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-1", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-1",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -567,7 +632,11 @@ def test_new_fixed_session(tmp_path: Path, project_test_data: Project) -> None:
     builder.session("session-2").connect()
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-2", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-2",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=None,
     )
 
 
@@ -605,7 +674,7 @@ def test_app_config(tmp_path: Path, project_test_data: Project) -> None:
     app_templates_resp = Mock()
     app_templates_resp.results = [_make_app_template()]
     app_templates_resp.metadata.next_paging_token = None
-    sdk.list_application_templates.return_value = app_templates_resp
+    sdk.list_app_configs.return_value = app_templates_resp
 
     build = _make_build()
     builds_resp = Mock()
@@ -635,12 +704,60 @@ def test_app_config(tmp_path: Path, project_test_data: Project) -> None:
     ) as f:
         data = yaml.safe_load(f)
 
-    assert data["docker"]["image"] == "localhost:5556/docker_image_name"
+    assert data["docker"]["image"] == "localhost:5555/docker_image_name"
     for nodes_type, node_config in data["available_node_types"].items():
         assert (
-            node_config["docker"]["worker_image"] == "localhost:5556/docker_image_name"
+            node_config["docker"]["worker_image"] == "localhost:5555/docker_image_name"
         )
 
     builder._up_session.assert_called_once_with(
-        ANY, "session-0", "anyscale_default_cloud", scratch_dir,
+        ANY,
+        "session-0",
+        "anyscale_default_cloud",
+        scratch_dir,
+        dangerously_set_build_id=build.id,
+    )
+
+
+def test_get_wheel_url() -> None:
+    wheel_prefix = (
+        "https://s3-us-west-2.amazonaws.com/ray-wheels/master/COMMIT_ID/ray-2.0.0.dev0"
+    )
+    assert (
+        _get_wheel_url("master/COMMIT_ID", "36", "darwin")
+        == f"{wheel_prefix}-cp36-cp36m-macosx_10_13_intel.whl"
+    )
+    assert (
+        _get_wheel_url("master/COMMIT_ID", "37", "darwin")
+        == f"{wheel_prefix}-cp37-cp37m-macosx_10_13_intel.whl"
+    )
+    assert (
+        _get_wheel_url("master/COMMIT_ID", "38", "darwin")
+        == f"{wheel_prefix}-cp38-cp38-macosx_10_13_x86_64.whl"
+    )
+
+    assert (
+        _get_wheel_url("master/COMMIT_ID", "36", "linux")
+        == f"{wheel_prefix}-cp36-cp36m-manylinux2014_x86_64.whl"
+    )
+    assert (
+        _get_wheel_url("master/COMMIT_ID", "37", "linux")
+        == f"{wheel_prefix}-cp37-cp37m-manylinux2014_x86_64.whl"
+    )
+    assert (
+        _get_wheel_url("master/COMMIT_ID", "38", "linux")
+        == f"{wheel_prefix}-cp38-cp38-manylinux2014_x86_64.whl"
+    )
+
+    assert (
+        _get_wheel_url("master/COMMIT_ID", "36", "win32")
+        == f"{wheel_prefix}-cp36-cp36m-win_amd64.whl"
+    )
+    assert (
+        _get_wheel_url("master/COMMIT_ID", "37", "win32")
+        == f"{wheel_prefix}-cp37-cp37m-win_amd64.whl"
+    )
+    assert (
+        _get_wheel_url("master/COMMIT_ID", "38", "win32")
+        == f"{wheel_prefix}-cp38-cp38-win_amd64.whl"
     )
