@@ -2,6 +2,8 @@ import json
 import os
 from unittest.mock import ANY, Mock, mock_open, patch
 
+import pytest
+
 from anyscale.client.openapi_client.models.project import Project  # type: ignore
 from anyscale.client.openapi_client.models.project_response import ProjectResponse  # type: ignore
 from anyscale.client.openapi_client.models.session import Session  # type: ignore
@@ -65,7 +67,8 @@ def test_get_proj_name_from_id(project_test_data: Project) -> None:
     )
 
 
-def test_get_proj_id_from_name(project_test_data: Project) -> None:
+@pytest.mark.parametrize("owner", [None, "owner"])
+def test_get_proj_id_from_name(project_test_data: Project, owner: str) -> None:
     mock_api_client = Mock()
     mock_api_client.find_project_by_project_name_api_v2_projects_find_by_name_get.return_value.results = [
         project_test_data
@@ -74,7 +77,7 @@ def test_get_proj_id_from_name(project_test_data: Project) -> None:
 
     assert project_id == project_test_data.id
     mock_api_client.find_project_by_project_name_api_v2_projects_find_by_name_get.assert_called_once_with(
-        name=project_test_data.name, _request_timeout=ANY,
+        name=project_test_data.name, _request_timeout=ANY, owner=None
     )
 
 
